﻿namespace Lab_3;

interface

uses
  System.Drawing,
  System.Collections,
  System.Collections.Generic,
  System.Windows.Forms,
  System.ComponentModel;

type
  /// <summary>
  /// Summary description for MainForm.
  /// </summary>
  MainForm = partial class(System.Windows.Forms.Form)
  private
    Socket: System.Net.Sockets.Socket;
    method ShowMessage(Msg: String);
    method Connect(sender: System.Object; e: System.EventArgs);
    method Disconnect(sender: System.Object; e: System.EventArgs);
    method Send(sender: System.Object; e: System.EventArgs);
    method TimeoutChange(sender: System.Object; e: System.EventArgs);
    method Refresh(sender: System.Object; e: System.EventArgs);
    method OnClose(sender: System.Object; e: System.Windows.Forms.FormClosedEventArgs);
    method eMessage_KeyUp(sender: System.Object; e: System.Windows.Forms.KeyEventArgs);
  protected
    method Dispose(disposing: Boolean); override;
  public
    constructor;
  end;

implementation

{$REGION Construction and Disposition}
constructor MainForm;
begin
  //
  // Required for Windows Form Designer support
  //
  InitializeComponent();

  //
  // TODO: Add any constructor code after InitializeComponent call
  //
end;

method MainForm.Dispose(disposing: Boolean);
begin
  if disposing then begin
    if assigned(components) then
      components.Dispose();

    //
    // TODO: Add custom disposition code here
    //
  end;
  inherited Dispose(disposing);
end;
{$ENDREGION}

method MainForm.Connect(sender: System.Object; e: System.EventArgs);
begin
  eLog.AppendText('System: Connecting...'#13);
  Socket:=new System.Net.Sockets.Socket(
    System.Net.Dns.GetHostEntry(System.Net.Dns.GetHostName).AddressList[0].AddressFamily,
    System.Net.Sockets.SocketType.Stream,
    System.Net.Sockets.ProtocolType.IP);
  try
    Socket.Connect(eHost.Text, 9999);
  except
    eLog.AppendText('System: Connect error'#13);
    exit;
  end;
  eLog.AppendText('System: Connected'#13);
  btnConnect.Enabled:=false;
  btnDisconnect.Enabled:=true;
  btnSend.Enabled:=true;
  case cbTimeout.SelectedIndex of
    0: Timer.Interval:=1000;
    1: Timer.Interval:=5000;
    2: Timer.Interval:=10000;
    3: Timer.Interval:=15000;
    4: Timer.Interval:=30000;
    5: Timer.Interval:=60000;
  else
    Timer.Interval:=5000;
  end; // case
  Timer.Start;
end;

method MainForm.Disconnect(sender: System.Object; e: System.EventArgs);
begin
  Socket.Close;
  eLog.AppendText('System: Disconnected'#13);
  btnConnect.Enabled:=true;
  btnDisconnect.Enabled:=false;
  btnSend.Enabled:=false;
  Timer.Stop;
end;

method MainForm.Send(sender: System.Object; e: System.EventArgs);
var
  Data: array of byte;
  Error: System.Net.Sockets.SocketError;
begin
  if eMessage.Text.Length<=0 then exit; 
  if not Socket.Connected then 
  begin
    eLog.AppendText('System: Disconnect'#13);
    btnConnect.Enabled:=true;
    btnDisconnect.Enabled:=false;
    btnSend.Enabled:=false; 
    Timer.Stop;
    exit;
  end;
  Data:= System.Text.Encoding.Default.GetBytes(eMessage.Text);
  try
    Socket.Send(Data, 0, Data.Length, System.Net.Sockets.SocketFlags.None);
    eLog.AppendText('Client: '+eMessage.Text+#13);
    eMessage.Text:='';
  except
    eLog.AppendText('System: Send error'#13);
    Socket.Close;
    eLog.AppendText('System: Disconnect'#13);
    btnConnect.Enabled:=true;
    btnDisconnect.Enabled:=false;
    btnSend.Enabled:=false; 
    Timer.Stop;
    exit;
  end;    
end;

method MainForm.TimeoutChange(sender: System.Object; e: System.EventArgs);
begin
  case cbTimeout.SelectedIndex of
    0: Timer.Interval:=1000;
    1: Timer.Interval:=5000;
    2: Timer.Interval:=10000;
    3: Timer.Interval:=15000;
    4: Timer.Interval:=30000;
    5: Timer.Interval:=60000;
  else
    Timer.Interval:=5000;
  end; 
end;

method MainForm.Refresh(sender: System.Object; e: System.EventArgs);
var
  data: array of byte;
  i: Integer;
  msg: String;
begin
  while Socket.Available>0 do
  begin
    try
      data:=new array [0..255] of byte;
      i:=Socket.Receive(data);
      msg:=System.Text.Encoding.Default.GetString(data);
      msg:=msg.Remove(i,msg.Length-i);
      eLog.AppendText(msg);
      eLog.AppendText(#13);
    except
      eLog.AppendText('System: Recieve error'#13);
    end;
  end;
  if Socket.Poll(100, System.Net.Sockets.SelectMode.SelectRead) then 
  begin
    eLog.AppendText('System: Disconnect'#13);
    btnConnect.Enabled:=true;
    btnDisconnect.Enabled:=false;
    btnSend.Enabled:=false; 
    Timer.Stop;
    exit;
  end;
end;

method MainForm.ShowMessage(Msg: String);
begin
  System.Windows.Forms.MessageBox.Show(Msg);
end;

method MainForm.OnClose(sender: System.Object; e: System.Windows.Forms.FormClosedEventArgs);
begin
  if Socket<>nil then if Socket.Connected then Socket.Close;
end;

method MainForm.eMessage_KeyUp(sender: System.Object; e: System.Windows.Forms.KeyEventArgs);
begin
  if e.KeyCode=Keys.Enter then Send(sender,e);
end;

end.
